const { gql } = require('apollo-server');

const typeDefs = gql`

    scalar IpAddress

   
    type ClusterStatus {
        status: CLUSTER_STATUS!
        message: String
    }
    
    type OntapClusterProperties {
        ontapVersion: String!
        clusterAddress: IpAddress!
    }
    
    enum CLUSTER_STATUS {
        FAILED, 
        ON,
        UNKNOWN
    }
    
    type PerformanceCapacity{
        inactiveCapacity: Float
        activeCapacity: Float
        isEstimatedInactiveCapacity: Boolean
        usedPerformanceCapacity: Float
    }
    
    
    type CapacityInformation{
        usedPerformanceAndTieredCapacity: Float 
        physicalCapacity: Float
        performanceCapacity: PerformanceCapacity
        savingOpportunities: Int 
    }
    
    type TierInformation{
        bucketName: String!
        providerType: String!
        tieredCapacity: Float!
        objectStoreName: String!
        regionName: String
        tierLevel: String
        savings: Int
    }

    type ClusterVolumes{
        tieredVolumes: Int,
        eligibleVolumes : Int
    }
    
    type Stats{
        tieredCapacity: Float!
        numberOfActiveClusters : Int!
        numberOfClusters: Int!
    }
    type Cluster {
        clusterId: ID!
        name: String
        ontapClusterProperties: OntapClusterProperties
        status: ClusterStatus!
        capacityInformation: CapacityInformation
        tierInformation : TierInformation
        volumes : ClusterVolumes,
        agentCloudProvider: CLOUD_PROVIDERS
        agentId: ID
    }

    type EmptyResult {
        status: String
    }

    input VolumeIdentifier {
        name: String!
        svmName: String!
    }
    
    input AwsKeys{
        accessKey: String,
        secretKey: String
    }

    input ProviderProperties {
        awsKeys : AwsKeys
        cloudProvider: CLOUD_PROVIDERS!,
        region: String!
        storageClass:STORAGE_CLASS
    }
    
    input DiscoverParams {
        clusterAddress: IpAddress!
        clusterUserName: String!
        clusterPassword: String!
        agentId: String!
    }

    input StartTieringParams {
        providerProperties : ProviderProperties!
        clusterId: ID!
        volumeIdentifiers: [VolumeIdentifier!]!
        tieringPolicy: INPUT_TIERING_POLICY,
        ipSpace:String!
    }
    
    enum STORAGE_CLASS{
        standard,
        standard_ia
        hot
    }

    input AddToTierParams {
        clusterId: ID!,
        volumeIdentifiers: [VolumeIdentifier!]!,
        tieringPolicy: INPUT_TIERING_POLICY
    }

    enum INPUT_TIERING_POLICY {
        auto,
        snapshot_only
    }
    
    enum TIERING_POLICY {
        none,
        auto,
        snapshot_only,
        backup
    }

    type Volume {
        name: String!
        svmName: String!
        aggregateName: String
        capacity: Float
        inactiveCapacity: Float
        inactiveCapacityPercent : Int
        tieringPolicy: TIERING_POLICY,
        tieredCapacity: Float
        isEligibleForTiering : Boolean!
        tieringRestrictionReason: String
        isEstimatedInactiveCapacity: Boolean!
        usedPerformanceCapacity : Float
        usedPerformanceAndTieredCapacity: Float
    }

    type Agent{
        agentId: String!,
        name: String,
        provider: CLOUD_PROVIDERS!,
        region: String!,
        network: String!,
        subnet: String!,
        creatorUserId: String!,
        status : AgentStatus!
        reason: String
    }
    
    type AccountInfo{
        accountId:  ID!
        subscriptionStatus:SubscriptionStatus
        daysLeftForTrial: Int
    }
    
    type AgentResponse {
        agents: [Agent]!,
        accountId: ID!
    }

    type Region {
        region: String!
        regionName: String
    }
    
    type DiscoverResponse {
        clusterId: ID!
    }
    
    

    enum AgentStatus {
        pending,
        active,
        inactive,
        unknown
    }

    enum CLOUD_PROVIDERS {
        AWS,
        AZURE
    }

    enum SubscriptionStatus{
        subscribed
        unsubscribed,
        failed,
        pending,
        unsubscribe_pending
    }

    type Query {
        "Get all agents of account"
        agents(agentId: ID): AgentResponse
        "Get all discovered clusters"
        clusters(clusterId: ID): [Cluster]!
        "Get cluster's tiering-supported volumes"
        volumes(clusterId: ID!): [Volume]!
        "Get AWS regions"
        regions(type: CLOUD_PROVIDERS!, regionCode: String, clusterId: ID): [Region]!,
        ipSpaces(clusterId: ID!):[String]
        accountInfo:AccountInfo
    }

    type Mutation{
        "Discover ONTAP cluster"
        discover(input : DiscoverParams) : DiscoverResponse!
        "Start tiering volumes"
        startTiering(input : StartTieringParams): EmptyResult
        "Tier more volumes"
        addToTier(input:AddToTierParams ): EmptyResult
    }`;

module.exports = typeDefs;
