const {
    CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX, CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX, CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX,
    OCCM_BASE_URL, CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX, DISCOVER_LOCATION,
    CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX
} = require('../../utils/consts');
const { validateResponse, validateRequest } = require('../../utils/schema-validation');
const {
    clusterMetadataResponseSchema, clusterResponseSchema, licenseStatusResponseSchema,
    licenseSchema, discoverRequestSchema, configureFabricPoolResourcesRequest, configureAzureFabricPoolResourcesRequest
} = require('../../joi-schemas/cloud-manager-schamas');
const getLogger = require('../../utils/logger/logger');
const {
    getRequest, postRequest, putRequest, deleteRequest
} = require('./cloud-manager-axios-utils');

const logger = getLogger();


async function clusterMetadata(clusterAddress, clusterUserName, clusterPassword, agentId) {
    logger.info(`cluster metadata for address ${clusterAddress} with clusterUserName ${clusterUserName} and agent ${agentId}`);
    const data = await getRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX}`,
        {
            clusterAddress,
            clusterUserName,
            clusterPassword,
            'X-Agent-Id': agentId
        });
    validateResponse(data, clusterMetadataResponseSchema);
    return data;
}

async function discoverCluster(name, clusterAddress, clusterUserName, clusterPassword, workspaceId, agentId) {
    const requestData = {
        tenantId: workspaceId, name, clusterAddress, clusterUserName, clusterPassword, location: DISCOVER_LOCATION
    };
    validateRequest(requestData, discoverRequestSchema);
    logger.info(`About to discover cluster with address ${clusterAddress} in workspace ${workspaceId} and agentId ${agentId}`);
    const data = await postRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, requestData, { 'X-Agent-Id': agentId });
    return data.publicId;
}

async function clusterInfo(clusterId) {
    try {
        const data = await getRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}?fields=ontapClusterProperties,capacityFeatures,status`);
        validateResponse(data, clusterResponseSchema);
        return data;
    } catch (err) {
        const errMessage = `Received error for get clusterInfo with id ${clusterId}, ${JSON.stringify(err)}`;
        if (err.status === 504) {
            logger.warn(errMessage);
        } else {
            logger.error(errMessage);
        }
        return {
            publicId: clusterId,
            status: {
                status: 'UNKNOWN',
                message: err.message
            }
        };
    }
}

async function applyLicense(clusterId, license) {
    validateRequest(license, licenseSchema);
    return postRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}/${CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX}`, license);
}

async function licenseStatus(clusterId) {
    const data = await getRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}/${CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX}`);
    validateResponse(data, licenseStatusResponseSchema);
    return data;
}

async function enableColdDataReporting(clusterId) {
    logger.info(`Enable clod data reporting for cluster ${clusterId}`);
    return putRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}/${CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX}`, undefined);
}

async function configureFabricPoolResources(clusterId, region, awsAccessKeys, tierLevel, ipSpace) {
    logger.info('About to configure fabric pool resources on clusterId', clusterId, 'on region', region, 'and ipSpace', ipSpace, 'and storage class', tierLevel);
    const requestData = {
        region, awsAccessKeys, tierLevel, ipSpace
    };
    validateRequest(requestData, configureFabricPoolResourcesRequest);
    return postRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}/${CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX}`, requestData);
}

async function configureAzureFabricPoolResources(clusterId, region, ipSpace) {
    logger.info('About to configure fabric pool resources on clusterId', clusterId, 'and ipSpace', ipSpace, 'on region', region);
    const requestData = { region, ipSpace };
    validateRequest(requestData, configureAzureFabricPoolResourcesRequest);
    return postRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}/${CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX}`, requestData);
}


async function removeCluster(clusterId) {
    logger.info(`remove cluster ${clusterId}`);
    return deleteRequest(`${OCCM_BASE_URL}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${clusterId}`);
}


module.exports = {
    clusterInfo,
    discoverCluster,
    clusterMetadata,
    enableColdDataReporting,
    licenseStatus,
    removeCluster,
    configureAzureFabricPoolResources,
    applyLicense,
    configureFabricPoolResources
};
