const { URL } = require('url');
const nock = require('nock');
const {
    last, nth, findIndex, cloneDeep
} = require('lodash');
const { validateSchema } = require('../../../src/utils/schema-validation');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_VOLUMES_PATH_PREFIX, SAVING_OPPORTUNITY_ESTIMATION_RATIO
} = require('../../../src/utils/consts');
const { volumeSchema, modifyVolumeRequestSchema } = require('../../../src/joi-schemas/cloud-manager-schamas');
const { escapeForwardSlash, convertBytesToCapacity, findUserId } = require('../simulator-utils');
const { STATE, USERS } = require('../simulator-state');
const getLogger = require('../../../src/utils/logger/logger');
const { capacityToBytes } = require('../../../src/utils/utils');

const logger = getLogger('simulator');

const volumesScope = nock(CLOUD_MANAGER_ENDPOINT)
    .log(evt => logger.info(evt))
    // create volumes
    .post(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/.+`), body => {
        try {
            validateSchema(body, volumeSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const volume = JSON.parse(requestBody);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }
        const requestUri = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const clusterId = last(requestUri.pathname.split('/'));

        if (USERS[userId].VOLUMES[clusterId]) {
            USERS[userId].VOLUMES[clusterId].push(volume);
        } else {
            USERS[userId].VOLUMES[clusterId] = [volume];
        }

        return volume;
    })

    // get volumes
    .get(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}`)
    .query(query => query.workingEnvironmentId)
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
            return [200, JSON.stringify([])];
        }
        const clusterId = requestURL.searchParams.get('workingEnvironmentId');
        const volumes = USERS[userId].VOLUMES[clusterId] ? USERS[userId].VOLUMES[clusterId] : [];

        return JSON.stringify(volumes);
    })

    // modify volume
    .put(escapeForwardSlash(`${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/.+/.+/.+`), body => {
        try {
            validateSchema(body, modifyVolumeRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(function (uri, requestBody) {
        const requestUri = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const pathNames = requestUri.pathname.split('/');
            const clusterId = nth(pathNames, 5);
            const svmName = nth(pathNames, 6);
            const volumeName = nth(pathNames, 7);
            const body = JSON.parse(requestBody);
            const { tieringPolicy } = body;
            if (USERS[userId].VOLUMES[clusterId]) {
                const volumesToModify = findIndex(USERS[userId].VOLUMES[clusterId], (volume => volume.name === volumeName && volume.svmName === svmName));
                if (volumesToModify > -1) {
                    USERS[userId].VOLUMES[clusterId][volumesToModify].tieringPolicy = tieringPolicy;

                    if (USERS[userId].CLUSTERS[clusterId]) {
                        const volumeTierInactiveUserDataBytes = USERS[userId].VOLUMES[clusterId][volumesToModify].tierInactiveUserData ? capacityToBytes(USERS[userId].VOLUMES[clusterId][volumesToModify].tierInactiveUserData) : capacityToBytes(USERS[userId].VOLUMES[clusterId][volumesToModify].usedSize) * SAVING_OPPORTUNITY_ESTIMATION_RATIO;
                        // add volume inactiveUserData to cluster object store used space
                        if (USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.objectStoreConfigExtended && USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.objectStoreConfigExtended.objectStoreConfig) {
                            USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace = convertBytesToCapacity(capacityToBytes(USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace) + volumeTierInactiveUserDataBytes);
                        }
                        // subtract volume inactiveUserData from cluster cold data
                        if (USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.inactiveUserData) {
                            USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.inactiveUserData = convertBytesToCapacity(capacityToBytes(USERS[userId].CLUSTERS[clusterId].cluster.capacityFeatures.inactiveUserData) - volumeTierInactiveUserDataBytes);
                        }
                    }
                    return [200, { volumeName }];
                }
            }
        }
        return [404];
    });


volumesScope.persist(true);
