const axios = require('axios');
const { signJwtToken } = require('../simulator-utils');
const { reset } = require('../simulator-state');
const {
    AGENTS_MGMT_ENDPOINT, API_PATH_AGENTS
} = require('../../../src/utils/consts');
require('../scopes/agents-scope');


const UUID_REGEX = /^[\d|\w]{8}-[\d\w]{4}-[\d\w]{4}-[\d\w]{4}-[\d\w]{12}$/;

const ACCOUNT_1_NAME = 'account-1';
const ACCOUNT_2_NAME = 'account-2';

const agent = {
    name: 'agent1',
    provider: 'aws',
    region: 'us-east-1',
    network: 'vpc-5b46c03e',
    subnet: 'subnet-f1f526a8',
    creatorUserId: 'userId',
    status: 'active'
};


const agentToCreate2 = {
    name: 'agent2',
    provider: 'aws',
    region: 'us-east-1',
    network: 'vpc-5b46c03e',
    subnet: 'subnet-f1f526a8',
    creatorUserId: 'userId',
    status: 'active'
};

describe('agents mgmt scope', () => {
    beforeEach(() => reset());

    test('agents - create single', async () => {
        const { data } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const expectedObject = Object.assign({}, { agentId: expect.stringMatching(UUID_REGEX) }, agent);
        expect({ agent: data }).toMatchObject({ agent: expectedObject });
    });

    test('agents - get single', async () => {
        const { data: postResponseData } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: getResponseData } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect({ agents: [postResponseData] }).toEqual(getResponseData);
    });

    test('agents - get 2 single agents different user', async () => {
        const { data: postAgentUser1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: getAgentUser1 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { data: postAgentUser2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: getAgentUser2 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        expect({ agents: [postAgentUser1] }).toEqual(getAgentUser1);
        expect({ agents: [postAgentUser2] }).toEqual(getAgentUser2);

        expect(getAgentUser1.agents).toHaveLength(1);
        expect(getAgentUser2.agents).toHaveLength(1);
    });

    test('agents - get non existent', async () => {
        await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual({ agents: [] });
    });

    test('agents - create an agent with one user try to get it with other user - should fail', async () => {
        try {
            await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
            await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
                headers: {
                    'X-User-Token': signJwtToken('user2'),
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(403);
        }
    });

    test('agents - create multiple agents in same workspace', async () => {
        const { data: agent1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agent2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agents } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(agents).toEqual({ agents: [agent1, agent2] });
    });

    test('agents - create multiple agents in same workspace with different users - make sure each user recives only his agents', async () => {
        const { data: agent1User1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agent2User1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agentsUser1 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agent1User2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent: agentToCreate2 }, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: agent2User2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent: agentToCreate2 }, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: agentsUser2 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });

        expect(agentsUser1).toEqual({ agents: [agent1User1, agent2User1] });
        expect(agentsUser2).toEqual({ agents: [agent1User2, agent2User2] });
    });

    test('agents - create multiple agents in different accounts', async () => {
        const { data: agent1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agent2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_2_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agentsAccount1 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agentsAccount2 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(agentsAccount1).toEqual({ agents: [agent1] });
        expect(agentsAccount2).toEqual({ agents: [agent2] });
    });

    test('agents - create multiple agents in different accounts and different users', async () => {
        const { data: agent1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agent2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_2_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agentsAccount1 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_1_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: agentsAccount2 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=${ACCOUNT_2_NAME}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { data: agent1User2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: 'account_3', agent }, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: agent2User2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: 'account_4', agent }, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: agentsAccount1User2 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=account_3`, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: agentsAccount2User2 } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}?account_id=account_4`, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });


        expect(agentsAccount1).toEqual({ agents: [agent1] });
        expect(agentsAccount2).toEqual({ agents: [agent2] });

        expect(agentsAccount1User2).toEqual({ agents: [agent1User2] });
        expect(agentsAccount2User2).toEqual({ agents: [agent2User2] });
    });

    test('get agent status- existing agent in the account', async () => {
        const { data: agent1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: { agent: agentsAccount1 } } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        expect(agentsAccount1.status).toEqual('active');
    });

    test('get agent status- existing agent in the account with multiple users', async () => {
        const { data: agent1 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: { agent: agentsAccount1 } } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });

        const { data: agent1User2 } = await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2')
            }
        });
        const { data: { agent: agentsAccount1User2 } } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/${agent1User2.agentId}?account_id=${ACCOUNT_1_NAME}`,
            {
                headers: {
                    'X-User-Token': signJwtToken('user2'),
                    authorization: signJwtToken('user2')
                }
            });
        expect(agentsAccount1.status).toEqual('active');
        expect(agentsAccount1User2.status).toEqual('active');
    });

    test('get agent status, non existing agent in an account', async () => {
        try {
            const { data } = await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/agentId?account_id=${ACCOUNT_1_NAME}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
            expect(data).toEqual([]);
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('get agent status, non existing agent in an account for user ', async () => {
        try {
            await axios.post(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}`, { accountId: ACCOUNT_1_NAME, agent }, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });


            await axios.get(`${AGENTS_MGMT_ENDPOINT}${API_PATH_AGENTS}/agentId?account_id=${ACCOUNT_1_NAME}`, {
                headers: {
                    'X-User-Token': signJwtToken('user2'),
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });
});
