const axios = require('axios');
const { signJwtToken } = require('../simulator-utils');
const { reset } = require('../simulator-state');
const {
    SUBSCRIPTION_ENDPOINT, MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS,
    SUBSCRIBED, UNSUBSCRIBED, TIERING_SERVICE_NAME_SUBCRIPTION
} = require('../../../src/utils/consts');
require('../scopes/subscription-service-scope');
require('../scopes/multi-tenancy-scope');


const ACCOUNT_1_ID = 'account-1';

const subscription = {
    subscriptionId: 'customerIdentifier1',
    cloudProvider: 'aws',
    name: 'subscription',
    subscriptionStatus: SUBSCRIBED,
    serviceName: TIERING_SERVICE_NAME_SUBCRIPTION
};


const subscription2 = {
    subscriptionId: 'customerIdentifier2',
    cloudProvider: 'aws',
    subscriptionStatus: UNSUBSCRIBED,
    name: 'subscription2',
    serviceName: TIERING_SERVICE_NAME_SUBCRIPTION
};

describe('subscription service scope', () => {
    beforeEach(() => reset());

    test('subscription - create subscription ', async () => {
        // create account
        const { data: account } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountId } = account;
        subscription.accounts = [accountId];
        // create subscription
        const { data } = await axios.post(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, subscription, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject(subscription);
    });

    test('subscription - list subscriptions', async () => {
        // create account
        const { data: account } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountId } = account;

        // create subscription
        const { data: subscriptionCreated } = await axios.post(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, subscription, {
            headers: {
                authorization: signJwtToken(),
                'X-User-Token': signJwtToken()
            }
        });

        const { data: listSubscriptionsResponse } = await axios.get(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(listSubscriptionsResponse).toEqual([subscriptionCreated]);
    });

    test('subscription - get empty array of subscriptions', async () => {
        // create account
        const { data: account } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { accountPublicId: accountId } = account;

        const { data: listSubscriptionsResponse } = await axios.get(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, {
            headers: {
                authorization: signJwtToken(),
                'X-User-Token': signJwtToken()
            }
        });

        expect(listSubscriptionsResponse).toEqual([]);
    });

    test('subscription - create multiple subscriptions in same account', async () => {
        // create account
        const { data: account } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountId } = account;

        // create subscription
        const { data: subscriptionCreated } = await axios.post(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, subscription, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { data: subscription2Created } = await axios.post(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, subscription2, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        const { data: listSubscriptionsResponse } = await axios.get(`${SUBSCRIPTION_ENDPOINT}/${accountId}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(listSubscriptionsResponse).toEqual([subscriptionCreated, subscription2Created]);
    });


    test('subscription - get subscription of non existent account', async () => {
        try {
            await axios.get(`${SUBSCRIPTION_ENDPOINT}/${ACCOUNT_1_ID}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('subscriptions- create a subscription with one user try to get it with other user - should an empty array', async () => {
        // create account user1
        const { data: accountUser1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_ID}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { accountPublicId: accountIdUser1 } = accountUser1;

        // create subscription
        await axios.post(`${SUBSCRIPTION_ENDPOINT}/${accountIdUser1}`, subscription, {
            headers: {
                authorization: signJwtToken()
            }
        });

        try {
            await axios.get(`${SUBSCRIPTION_ENDPOINT}/${accountIdUser1}`, {
                headers: {
                    authorization: signJwtToken('user2'),
                    'X-User-Token': signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toEqual(404);
        }
    });
});
