const axios = require('axios');
const uuidv4 = require('uuid/v4');
const { reset } = require('../simulator-state');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_VOLUMES_PATH_PREFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');

require('../scopes/volumes-scope');


const volume = {
    name: 'volumeName',
    svmName: 'svm_1',
    aggregateName: 'aggr-1',
    size: { size: 10, unit: 'TB' },
    usedSize: { size: 5, unit: 'TB' },
    tieringPolicy: 'auto'
};

const volume2 = {
    name: 'vol2',
    svmName: 'svm_1',
    aggregateName: 'aggr-1',
    size: { size: 10, unit: 'TB' },
    usedSize: { size: 5, unit: 'TB' },
    tieringPolicy: 'auto'
};

describe('volume scope', () => {
    beforeEach(() => reset('VOLUMES'));

    test('create volume', async () => {
        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject(volume);
    });

    test('get volumes when there  are no volumes ', async () => {
        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=cluster_1`, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject([]);
    });

    test('get volumes when there is a volumes ', async () => {
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster_1`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=cluster_1`, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject([volume]);
    });

    test('create/get volumes of different users- each user should receive its own volumes', async () => {
        const clusterId = uuidv4();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${clusterId}`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data: volumeUser1 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${clusterId}`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${clusterId}`, volume2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });
        const { data: volumeUser2 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${clusterId}`, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        expect(volumeUser1).toMatchObject([volume]);
        expect(volumeUser2).toMatchObject([volume2]);
    });

    test('modify volumes - modify for non existing volume ', async () => {
        try {
            await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster_1/svm_name/volumeName`,
                { tieringPolicy: 'auto' }, {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('modify volumes - modify for existing volume ', async () => {
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster_1`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster_1/svm_1/volumeName`,
            { tieringPolicy: 'auto' }, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(data.volumeName).toEqual('volumeName');
    });

    test('modify volumes- try to modify a volume of a different user', async () => {
        try {
            const clusterId = uuidv4();

            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${clusterId}`, volume, {
                headers: {
                    authorization: signJwtToken()
                }
            });

            await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster_1/svm_1/volumeName`,
                { tieringPolicy: 'auto' }, {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.status).toEqual(404);
        }
    });
});
